gnunet-svn
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[taler-exchange] branch master updated: enable auditor disable client lo


From: gnunet
Subject: [taler-exchange] branch master updated: enable auditor disable client logic
Date: Wed, 25 Nov 2020 21:29:07 +0100

This is an automated email from the git hooks/post-receive script.

grothoff pushed a commit to branch master
in repository exchange.

The following commit(s) were added to refs/heads/master by this push:
     new 4d9b672a enable auditor disable client logic
4d9b672a is described below

commit 4d9b672a720f77d9da4fcdba07aa891f4a6a9672
Author: Christian Grothoff <christian@grothoff.org>
AuthorDate: Wed Nov 25 21:29:04 2020 +0100

    enable auditor disable client logic
---
 src/include/taler_exchange_service.h              |   6 +-
 src/lib/Makefile.am                               |   1 +
 src/lib/exchange_api_management_auditor_disable.c | 162 +++++++++++++++++++---
 3 files changed, 147 insertions(+), 22 deletions(-)

diff --git a/src/lib/Makefile.am b/src/lib/Makefile.am
index 4c447c18..bf0ef6f4 100644
--- a/src/lib/Makefile.am
+++ b/src/lib/Makefile.am
@@ -27,6 +27,7 @@ libtalerexchange_la_SOURCES = \
   exchange_api_deposit.c \
   exchange_api_deposits_get.c \
   exchange_api_link.c \
+  exchange_api_management_auditor_disable.c \
   exchange_api_management_revoke_signing_key.c \
   exchange_api_melt.c \
   exchange_api_recoup.c \
diff --git a/src/lib/exchange_api_management_auditor_disable.c 
b/src/lib/exchange_api_management_auditor_disable.c
index 8cd4e322..b70a8946 100644
--- a/src/lib/exchange_api_management_auditor_disable.c
+++ b/src/lib/exchange_api_management_auditor_disable.c
@@ -24,9 +24,11 @@
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_exchange_service.h"
 #include "taler_signatures.h"
+#include "taler_curl_lib.h"
+#include "taler_json_lib.h"
 
 /**
- * @brief Handle for a POST /management/auditors/disable request.
+ * @brief Handle for a POST /management/auditors/$AUDITOR_PUB/disable request.
  */
 struct TALER_EXCHANGE_ManagementAuditorDisableHandle
 {
@@ -36,6 +38,11 @@ struct TALER_EXCHANGE_ManagementAuditorDisableHandle
    */
   char *url;
 
+  /**
+   * Minor context that holds body and headers.
+   */
+  struct TALER_CURL_PostContext post_ctx;
+
   /**
    * Handle for the request.
    */
@@ -57,37 +64,153 @@ struct TALER_EXCHANGE_ManagementAuditorDisableHandle
   struct GNUNET_CURL_Context *ctx;
 };
 
-
 /**
- * Inform the exchange that an auditor should be disabled.
+ * Function called when we're done processing the
+ * HTTP /management/signkeys/%s/revoke request.
  *
- * @param ctx the context
- * @param url HTTP base URL for the exchange
- * @param auditor_pub the public signing key of the auditor
- * @param auditor_url base URL of the auditor
- * @param validity_start when was this decided?
- * @param master_sig signature affirming the auditor addition
- * @param cb function to call with the exchange's result
- * @param cb_cls closure for @a cb
- * @return the request handle; NULL upon error
+ * @param cls the `struct TALER_EXCHANGE_ManagementAuditorDisableHandle *`
+ * @param response_code HTTP response code, 0 on error
+ * @param response response body, NULL if not in JSON
  */
+static void
+handle_auditor_disable_finished (void *cls,
+                                 long response_code,
+                                 const void *response)
+{
+  struct TALER_EXCHANGE_ManagementAuditorDisableHandle *ah = cls;
+  const json_t *json = response;
+  struct TALER_EXCHANGE_HttpResponse hr = {
+    .http_status = (unsigned int) response_code,
+    .reply = json
+  };
+
+  ah->job = NULL;
+  switch (response_code)
+  {
+  case MHD_HTTP_NO_CONTENT:
+    break;
+  case MHD_HTTP_FORBIDDEN:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_NOT_FOUND:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_CONFLICT:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  default:
+    /* unexpected response code */
+    GNUNET_break_op (0);
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) hr.ec);
+    break;
+  }
+  if (NULL != ah->cb)
+  {
+    ah->cb (ah->cb_cls,
+            &hr);
+    ah->cb = NULL;
+  }
+  TALER_EXCHANGE_management_disable_auditor_cancel (ah);
+}
+
+
 struct TALER_EXCHANGE_ManagementAuditorDisableHandle *
 TALER_EXCHANGE_management_disable_auditor (
   struct GNUNET_CURL_Context *ctx,
   const char *url,
   const struct TALER_AuditorPublicKeyP *auditor_pub,
   const char *auditor_url,
-  struct GNUNET_TIME_Absolute validity_start,
+  struct GNUNET_TIME_Absolute validity_end,
   const struct TALER_MasterSignatureP *master_sig,
   TALER_EXCHANGE_ManagementAuditorDisableCallback cb,
-  void *cb_cls);
+  void *cb_cls)
+{
+  struct TALER_EXCHANGE_ManagementAuditorDisableHandle *ah;
+  CURL *eh;
+  json_t *body;
+
+  ah = GNUNET_new (struct TALER_EXCHANGE_ManagementAuditorDisableHandle);
+  ah->cb = cb;
+  ah->cb_cls = cb_cls;
+  ah->ctx = ctx;
+  {
+    char epub_str[sizeof (*auditor_pub) * 2];
+    char arg_str[sizeof (epub_str) + 64];
+    char *end;
+
+    end = GNUNET_STRINGS_data_to_string (auditor_pub,
+                                         sizeof (*auditor_pub),
+                                         epub_str,
+                                         sizeof (epub_str));
+    *end = '\0';
+    GNUNET_snprintf (arg_str,
+                     sizeof (arg_str),
+                     "management/auditors/%s/disable",
+                     epub_str);
+    ah->url = TALER_url_join (url,
+                              arg_str,
+                              NULL);
+  }
+  if (NULL == ah->url)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    GNUNET_free (ah);
+    return NULL;
+  }
+  body = json_pack ("{s:o, s:o}",
+                    "master_sig",
+                    GNUNET_JSON_from_data_auto (master_sig),
+                    "validity_end",
+                    GNUNET_JSON_from_time_abs (validity_end));
+  if (NULL == body)
+  {
+    GNUNET_break (0);
+    GNUNET_free (ah->url);
+    GNUNET_free (ah);
+    return NULL;
+  }
+  eh = curl_easy_init ();
+  if (GNUNET_OK !=
+      TALER_curl_easy_post (&ah->post_ctx,
+                            eh,
+                            body))
+  {
+    GNUNET_break (0);
+    json_decref (body);
+    GNUNET_free (ah->url);
+    GNUNET_free (eh);
+    return NULL;
+  }
+  json_decref (body);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Requesting URL '%s'\n",
+              ah->url);
+  GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
+                                               CURLOPT_URL,
+                                               ah->url));
+  ah->job = GNUNET_CURL_job_add2 (ctx,
+                                  eh,
+                                  ah->post_ctx.headers,
+                                  &handle_auditor_disable_finished,
+                                  ah);
+  if (NULL == ah->job)
+  {
+    TALER_EXCHANGE_management_disable_auditor_cancel (ah);
+    return NULL;
+  }
+  return ah;
+}
 
 
-/**
- * Cancel #TALER_EXCHANGE_management_disable_auditor() operation.
- *
- * @param ah handle of the operation to cancel
- */
 void
 TALER_EXCHANGE_management_disable_auditor_cancel (
   struct TALER_EXCHANGE_ManagementAuditorDisableHandle *ah)
@@ -97,6 +220,7 @@ TALER_EXCHANGE_management_disable_auditor_cancel (
     GNUNET_CURL_job_cancel (ah->job);
     ah->job = NULL;
   }
+  TALER_curl_easy_post_finished (&ah->post_ctx);
   GNUNET_free (ah->url);
   GNUNET_free (ah);
 }

diff --git a/src/include/taler_exchange_service.h 
b/src/include/taler_exchange_service.h
index fa1cfde4..bb9b947a 100644
--- a/src/include/taler_exchange_service.h
+++ b/src/include/taler_exchange_service.h
@@ -2362,7 +2362,7 @@ typedef void
 
 
 /**
- * @brief Handle for a POST /management/auditors/disable request.
+ * @brief Handle for a POST /management/auditors/$AUDITOR_PUB/disable request.
  */
 struct TALER_EXCHANGE_ManagementAuditorDisableHandle;
 
@@ -2374,7 +2374,7 @@ struct TALER_EXCHANGE_ManagementAuditorDisableHandle;
  * @param url HTTP base URL for the exchange
  * @param auditor_pub the public signing key of the auditor
  * @param auditor_url base URL of the auditor
  * @param validity_start when was this decided?
  * @param master_sig signature affirming the auditor addition
  * @param cb function to call with the exchange's result
  * @param cb_cls closure for @a cb
@@ -2386,7 +2386,7 @@ TALER_EXCHANGE_management_disable_auditor (
   const char *url,
   const struct TALER_AuditorPublicKeyP *auditor_pub,
   const char *auditor_url,
-  struct GNUNET_TIME_Absolute validity_start,
+  struct GNUNET_TIME_Absolute validity_end,
   const struct TALER_MasterSignatureP *master_sig,
   TALER_EXCHANGE_ManagementAuditorDisableCallback cb,
   void *cb_cls);
@@ -2374,7 +2374,7 @@ struct TALER_EXCHANGE_ManagementAuditorDisableHandle;
  * @param auditor_pub the public signing key of the auditor
  * @param auditor_url base URL of the auditor
- * @param validity_start when was this decided?
+ * @param validity_end when was this decided?
  * @param master_sig signature affirming the auditor addition
  * @param cb function to call with the exchange's result
-- 
To stop receiving notification emails like this one, please contact
gnunet@gnunet.org.



reply via email to

[Prev in Thread] Current Thread [Next in Thread]